package com.reactnativeactivityrecognitionwrapper

import LocationHelper
import SensorInfo
import SensorRequestItem
import android.content.*
import android.os.Build
import android.util.Log
import android.widget.Toast
import androidx.annotation.RequiresApi
import androidx.localbroadcastmanager.content.LocalBroadcastManager
import com.facebook.react.bridge.*
import com.facebook.react.modules.core.DeviceEventManagerModule
import com.google.android.gms.location.DetectedActivity
import com.reactnativeactivityrecognitionwrapper.services.BackgroundDetectedActivitiesService
import com.reactnativeactivityrecognitionwrapper.types.LocationObj
import com.reactnativeactivityrecognitionwrapper.utils.Constants
import java.time.Instant
import java.time.ZoneId
import java.time.format.DateTimeFormatter.ofPattern
import java.util.*
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken

class ActivityRecognitionWrapperModule(reactContext: ReactApplicationContext) :
  ReactContextBaseJavaModule(reactContext) {
  private var accuracy: Number = 10
  private var updateDuration: Int = 100000
  private var distanceFilter: Int = 100
  private var notificationTitle: String = "Service Active"
  private var notificationDescription: String = "Location and activity reporting is active."

  override fun getName(): String {
    return "ActivityRecognitionWrapper"
  }

  var broadcastReceiver =
    object : BroadcastReceiver() {
      @RequiresApi(Build.VERSION_CODES.O)
      override fun onReceive(context: Context, intent: Intent) {
        if (intent.action == Constants.BROADCAST_DETECTED_ACTIVITY) {
          println("GOTACTIVITY - Broadcasting to RN")
          val type = intent.getIntExtra("type", -1)
          val confidence = intent.getIntExtra("confidence", 0)
          var timestamp = intent.getStringExtra("timestamp")
          if (timestamp == null) {
            timestamp =
              ofPattern("yyyy-MM-dd HH:mm:ss.SSSSSS Z")
                .withZone(ZoneId.systemDefault())
                .format(Instant.now())
                .toString()
          }

          provideUserStateOutput(timestamp, type, confidence)
        } else if (intent.action == Constants.BROADCAST_DETECTED_LOCATION) {
          Log.i("LOC RECEIVED - WRAPPER", intent.action!!)
          prepareLocationUpdateBroadcastAndSend(
            (intent.getSerializableExtra("locationObj") as LocationObj)
          )
        }
      }
    }

  init {
    var filter = IntentFilter()
    filter.addAction(Constants.BROADCAST_DETECTED_ACTIVITY)
    filter.addAction(Constants.BROADCAST_DETECTED_LOCATION)
    LocalBroadcastManager.getInstance(reactApplicationContext)
      .registerReceiver(broadcastReceiver, filter)


  }

  var intent = Intent(reactApplicationContext, BackgroundDetectedActivitiesService::class.java)

  private fun provideUserStateOutput(timestamp: String, type: Int, confidence: Int) {
    var label = "UNKNOWN"

    when (type) {
      DetectedActivity.IN_VEHICLE -> {
        label = "IN_VEHICLE"
      }
      DetectedActivity.ON_BICYCLE -> {
        label = "CYCLING"
      }
      DetectedActivity.ON_FOOT -> {
        label = "ON_FOOT"
      }
      DetectedActivity.RUNNING -> {
        label = "RUNNING"
      }
      DetectedActivity.STILL -> {
        label = "STILL"
      }
      DetectedActivity.TILTING -> {
        label = "TILTING"
      }
      DetectedActivity.WALKING -> {
        label = "WALKING"
      }
      DetectedActivity.UNKNOWN -> {
        label = "UNKNOWN"
      }
    }
    Log.i("ActivityDetectionLabel", "$timestamp: $type and $label")
    Log.i("ActivityDetection", "$timestamp: User activity: ${label}, Confidence: $confidence")
    prepareActivityUpdateBroadcastAndSend(timestamp, label, confidence.toString())
  }

  fun prepareActivityUpdateBroadcastAndSend(timestamp: String, label: String, confidence: String) {
    val params =
      Arguments.createMap().apply {
        putString("TYPE", label)
        putString("CONFIDENCE", confidence)
        putString("TIMESTAMP", timestamp)
      }

    sendEvent(reactApplicationContext, "ACTIVITY_UPDATE", params)
    Toast.makeText(reactApplicationContext, "$timestamp $label $confidence", Toast.LENGTH_LONG)
  }

  fun prepareLocationUpdateBroadcastAndSend(location: LocationObj) {
    Log.i("LOC_BROADCAST", "Sending location ${location}")
    sendEvent(reactApplicationContext, "LOCATION_UPDATE", location.getMap())
  }

  // Example method
  // See https://reactnative.dev/docs/native-modules-android
  @ReactMethod
  fun multiply(a: Int, b: Int, promise: Promise) {
    promise.resolve(a * b)
  }

  @ReactMethod
  fun enableActivityRecognitionTracking(
    distanceFilter: Int,
    updateDuration: Int,
    accuracy: Int,
    notificationTitle: String,
    notificationDescription: String,
    promise: Promise?
  ) {
    try {
//      val reactContext = (ReactBridge.getContext() as ReactApplicationContext)
      this.distanceFilter = distanceFilter
      this.updateDuration = updateDuration
      this.accuracy = accuracy
      this.notificationTitle = notificationTitle
      this.notificationDescription = notificationDescription
      intent.putExtra("DISTANCE_FILTER", distanceFilter)
      intent.putExtra("UPDATE_DURATION", updateDuration)
      intent.putExtra("ACCURACY", accuracy)
      intent.putExtra("NOTIFICATIONTITLE", notificationTitle)
      intent.putExtra("NOTIFICATIONDESCRIPTION", notificationDescription)

      intent.putExtra(Constants.LOCATION_TRACKING_TYPE, Constants.FREQUENT_LOCATION_UPDATES)

      val sharedPreferences = reactApplicationContext.getSharedPreferences(
        "REACT_NATIVE_ACTIVITY_RECOGNITION_PREFERENCES",
        Context.MODE_PRIVATE
      )
      val editor = sharedPreferences.edit()
      editor.putInt("DISTANCE_FILTER", distanceFilter)
      editor.putInt("UPDATE_DURATION", updateDuration)
      editor.putInt("ACCURACY", accuracy)
      editor.putString("NOTIFICATIONTITLE", notificationTitle)
      editor.putString("NOTIFICATIONDESCRIPTION", notificationDescription)
      editor.putString(Constants.LOCATION_TRACKING_TYPE, Constants.FREQUENT_LOCATION_UPDATES)
      editor.commit()
      promise?.resolve(true)
    } catch (e: Error) {
      Log.e("ActivityRecognition", e.toString())
      promise?.reject(e)
    }
  }


  @RequiresApi(Build.VERSION_CODES.O)
  @ReactMethod
  fun startFrequentLocationUpdates(
    promise: Promise
  ) {
    if (this::distanceFilter == null || updateDuration == null || accuracy == null) {
      Log.i(
        "ActivityRecogWrapper",
        "Distance Filter, Update Duration or Accuracy not initialised correctly - unable to start activity tracking service."
      )
    } else {

      Log.i("ActivityRecogWrapper", "STARTING FREQUENT LOCATION UPDATES.")
      reactApplicationContext.startForegroundService(intent)
      promise.resolve(true)
    }
  }

  @RequiresApi(Build.VERSION_CODES.O)
  @ReactMethod
  fun startSignificantLocationUpdates(promise: Promise) {
    var intent = Intent(reactApplicationContext, BackgroundDetectedActivitiesService::class.java)
    Log.i("ActivityRecogWrapper", "STARTING SIGNIFICANT LOCATION UPDATES.")

    intent.putExtra("DISTANCE_FILTER", distanceFilter)
    intent.putExtra("UPDATE_DURATION", updateDuration)
    intent.putExtra("ACCURACY", accuracy)
    intent.putExtra(Constants.LOCATION_TRACKING_TYPE, Constants.SIGNIFICANT_LOCATION_UPDATES)

    reactApplicationContext.startForegroundService(intent)
    promise.resolve(true)
  }

  @RequiresApi(Build.VERSION_CODES.O)
  @ReactMethod
  fun stop(promise: Promise) {
    var intent = Intent(reactApplicationContext, BackgroundDetectedActivitiesService::class.java)
    reactApplicationContext.stopService(intent)
    promise.resolve(true)
  }

  @RequiresApi(Build.VERSION_CODES.O)
  @ReactMethod
  fun getLastKnownLocation(promise: Promise) {
    var loc = LocationHelper.getLastKnownLocation(reactApplicationContext)
    Log.i("MODULE", "Got location ${loc.toString()}")

    loc?.let {
      var location = LocationHelper.getTransformedLocationObject(it)
      prepareLocationUpdateBroadcastAndSend(location)
    }
  }

  @ReactMethod
  fun addListener(eventName: String) {
    Log.i("ACTIVITYRECOGNITION", "Added listener $eventName")
  }

  @ReactMethod
  fun removeListeners(count: Int) {
    Log.i("ACTIVITYRECOGNITION", "Removed $count listeners")
  }


  @ReactMethod
  private fun startSensors(folderStructure: String, sensorInfoListString: String) {
    Log.i("SENSORSERVICE", "Starting sensors...")

    val intent = Intent("com.reactnativeactivityrecognitionwrapper.ACTION_ACTIVATE_SENSORS")
    val sensorFinalList = object : TypeToken<List<SensorRequestItem>>() {}.type
    val sensorInfoList: List<SensorRequestItem> = Gson().fromJson(sensorInfoListString, sensorFinalList)

    intent.putExtra("folderStructure", folderStructure)
    intent.putExtra("sensorInfoList", ArrayList(sensorInfoList)) // Convert to ArrayList for serialization
    LocalBroadcastManager.getInstance(reactApplicationContext).sendBroadcast(intent)
  }

  @ReactMethod
  private fun stopSensors() {
    val intent = Intent("com.reactnativeactivityrecognitionwrapper.ACTION_DEACTIVATE_SENSORS")
    LocalBroadcastManager.getInstance(reactApplicationContext).sendBroadcast(intent)
  }

  private fun sendEvent(reactContext: ReactContext, eventName: String, params: WritableMap?) {
    reactContext
      .getJSModule(DeviceEventManagerModule.RCTDeviceEventEmitter::class.java)
      .emit(eventName, params)
    Log.i("SENDEVENTMODULE", "Sent activity update $eventName")
  }
}
