import Foundation
import CoreMotion
import CoreLocation

@objc(ActivityRecognitionWrapper)
class ActivityRecognitionWrapper: RCTEventEmitter, CLLocationManagerDelegate{
    let motionActivityManager = CMMotionActivityManager()
    let motionSensorManager = CMMotionManager()
    var locationManager:CLLocationManager = CLLocationManager()
    var distanceFilter:Double = 50
    var updateDuration: Int = 300000
    var accuracy: Int = 100
    var activityServiceRunning: Bool = false
    public static var shared:ActivityRecognitionWrapper?
    
    override init() {
        super.init()
        ActivityRecognitionWrapper.shared = self
    }
    
    
    @objc(enableActivityRecognitionTracking:withUpdateDuration:withAccuracy:withResolver:withRejecter:) func enableActivityRecognitionTracking(_ distanceFilter: Double,updateDuration: Int, accuracy: Int, resolve:RCTPromiseResolveBlock,reject:RCTPromiseRejectBlock) -> Void {
        locationManager.delegate = self
        self.distanceFilter = distanceFilter
        self.updateDuration = updateDuration
        self.accuracy = accuracy
        locationManager.allowsBackgroundLocationUpdates=true
        locationManager.distanceFilter = distanceFilter
        locationManager.desiredAccuracy = getAccuracy(acc: accuracy)
        debugPrint("LOCATION ACCURACY CHANGED: \(accuracy): \(locationManager.desiredAccuracy)")
        
        locationManager.requestAlwaysAuthorization()
        resolve(true)
    }
    
    
    
    
    @objc(getLastKnownLocation:reject:)
    func getLastKnownLocation(_ resolve: RCTPromiseResolveBlock, reject: RCTPromiseRejectBlock) -> Void{
        debugPrint("Requested one location.")
        locationManager.requestLocation()
        resolve(true)
    }
    
    @objc(startSignificantLocationUpdates:reject:)
    func startSignificantLocationUpdates(_ resolve: RCTPromiseResolveBlock, reject: RCTPromiseRejectBlock) -> Void{
        debugPrint("Starting significant location tracking")
        locationManager.allowsBackgroundLocationUpdates=true
        locationManager.pausesLocationUpdatesAutomatically = false
        locationManager.desiredAccuracy = kCLLocationAccuracyHundredMeters
        locationManager.startMonitoringSignificantLocationChanges()
        resolve(true)
    }
    
    @objc(startFrequentLocationUpdates:reject:)
    func startFrequentLocationUpdates(_ resolve: RCTPromiseResolveBlock, reject: RCTPromiseRejectBlock) -> Void{
        debugPrint("Starting frequent location tracking")
        debugPrint("Distance Filter: \(distanceFilter)")
        locationManager.desiredAccuracy = kCLLocationAccuracyHundredMeters
        locationManager.pausesLocationUpdatesAutomatically = false
        locationManager.distanceFilter = distanceFilter
        locationManager.allowsBackgroundLocationUpdates=true
        locationManager.startUpdatingLocation()
        resolve(true)
    }
    
    @objc(stop:reject:)
    func stop(_ resolve: RCTPromiseResolveBlock, reject: RCTPromiseRejectBlock) -> Void{
        debugPrint("Stopping all location tracking")
        locationManager.stopUpdatingLocation()
        locationManager.stopMonitoringSignificantLocationChanges()
        resolve(true)
    }
    
    
    @objc(startSensors:resolve:reject:)
    func startSensors(_ samplingRate:Double, resolve: RCTPromiseResolveBlock, reject: RCTPromiseRejectBlock) -> Void{
        debugPrint("Starting sensors...")
        
        if motionSensorManager.isAccelerometerAvailable {
            motionSensorManager.accelerometerUpdateInterval = 1 // Update interval in seconds
            motionSensorManager.startAccelerometerUpdates(to: .main) { (accelerometerData, error) in
                guard let accelerometerData = accelerometerData else { return }
                print("Accelerometer data: \(accelerometerData.acceleration)")
                let dateFormatter = DateFormatter();
                dateFormatter.dateFormat = "yyyy-MM-dd HH:mm:ss.SSS Z";
                
                let accelerationX = accelerometerData.acceleration.x
                let accelerationY = accelerometerData.acceleration.y
                let accelerationZ = accelerometerData.acceleration.z
                let bootTime = Date(timeIntervalSinceNow: -ProcessInfo.processInfo.systemUptime)
                let timestamp = bootTime.timeIntervalSince1970 + accelerometerData.timestamp

                ActivityRecognitionWrapper.shared?.sendEvent(withName: "ACC",   body:["TIMESTAMP":timestamp, "X":accelerationX, "Y":accelerationY, "Z":accelerationZ ])
                
            }
        } else {
            print("Accelerometer not available.")
        }
        
        if motionSensorManager.isGyroAvailable {
            motionSensorManager.gyroUpdateInterval = 1 // Update interval in seconds
            motionSensorManager.startGyroUpdates(to: .main) { (data, error) in
                guard let data = data else { return }
                print("Gyroscope data: \(data.rotationRate)")
            }
        } else {
            print("Gyroscope not available.")
        }
        
        if motionSensorManager.isMagnetometerAvailable {
            motionSensorManager.magnetometerUpdateInterval = 1 // Update interval in seconds
            motionSensorManager.startMagnetometerUpdates(to: .main) { (data, error) in
                guard let data = data else { return }
                print("Magnetometer data: \(data.magneticField)")
            }
        } else {
            print("Magnetometer not available.")
        }
        resolve(true)
    }
    
    @objc(stopSensors:reject:)
    func stopSensors(resolve: RCTPromiseResolveBlock, reject: RCTPromiseRejectBlock) {
        if motionSensorManager.isAccelerometerActive {
            motionSensorManager.stopAccelerometerUpdates()
        }
        if motionSensorManager.isGyroActive {
            motionSensorManager.stopGyroUpdates()
        }
        if motionSensorManager.isMagnetometerActive {
            motionSensorManager.stopMagnetometerUpdates()
        }
    }
    
    func getConfidence(conf:CMMotionActivityConfidence?) -> String{
        if conf == CMMotionActivityConfidence.low {
            return "Low"
        } else if conf == CMMotionActivityConfidence.medium {
            return "Good"
        } else if conf == CMMotionActivityConfidence.high {
            return "High"
        } else {
            return "UNKNOWN"
        }
    }
    
    func getAccuracy(acc:Int?) -> CLLocationAccuracy{
        if acc == 0 {
            return kCLLocationAccuracyBest
        } else if acc == 1 {
            return kCLLocationAccuracyBestForNavigation
        } else if acc == 10 {
            return kCLLocationAccuracyNearestTenMeters
        } else if acc == 100 {
            return kCLLocationAccuracyHundredMeters
        }
        else if acc == 1000 {
            return kCLLocationAccuracyKilometer
        }
        else if acc == 3000 {
            return kCLLocationAccuracyThreeKilometers
        }
        else{
            return kCLLocationAccuracyKilometer
        }
    }
    
    func getTransformedActivityObject(activity: CMMotionActivity, type:String) -> [String:String] {
        let dateFormatter = DateFormatter();
        dateFormatter.dateFormat = "yyyy-MM-dd HH:mm:ss.SSS Z";
        return ["TIMESTAMP": dateFormatter.string(from: (activity.startDate)), "CONFIDENCE": getConfidence(conf: activity.confidence), "TYPE":type]
    }
    
    func getTransformedLocationObject(location: CLLocation) -> [String:String] {
        let dateFormatter = DateFormatter();
        dateFormatter.dateFormat = "yyyy-MM-dd HH:mm:ss.SSS Z";
        if #available(iOS 13.4, *) {
            return ["TIMESTAMP": dateFormatter.string(from: (location.timestamp)),
                    "LAT":location.coordinate.latitude.description,
                    "LONG":location.coordinate.longitude.description,
                    "HORIZONTAL_ACCURACY":location.horizontalAccuracy.description,
                    "VERTICAL_ACCURACY":location.verticalAccuracy.description,
                    "SPEED_ACCURACY":location.speedAccuracy.description,
                    "SPEED":location.speed.description,
                    "ALTITUDE":location.altitude.description,
                    "COURSE":location.course.description,
                    "COURSE_ACCURACY":location.courseAccuracy.description
            ]
        } else {
            // Fallback on earlier versions
            //courseAccuracy not available pre ios13.4
            return ["TIMESTAMP": dateFormatter.string(from: (location.timestamp)),
                    "LAT":location.coordinate.latitude.description,
                    "LONG":location.coordinate.longitude.description,
                    "HORIZONTAL_ACCURACY":location.horizontalAccuracy.description,
                    "VERTICAL_ACCURACY":location.verticalAccuracy.description,
                    "SPEED_ACCURACY":location.speedAccuracy.description,
                    "SPEED":location.speed.description,
                    "ALTITUDE":location.altitude.description,
                    "COURSE":location.course.description,
            ]
        }
    }
    
    
    override func supportedEvents() -> [String]! {
        return ["ACTIVITY_UPDATE", "LOCATION_UPDATE", "RESET_ACTIVITY_RECOGNITION", "ACC"]
    }
    
    
    @objc func locationManager(_ manager: CLLocationManager, didChangeAuthorization status: CLAuthorizationStatus) {
        debugPrint("Got status")
        switch status {
        case .denied: // Setting option: Never
            print("LocationManager didChangeAuthorization denied")
        case .notDetermined: // Setting option: Ask Next Time
            print("LocationManager didChangeAuthorization notDetermined")
        case .authorizedWhenInUse: // Setting option: While Using the App
            print("LocationManager didChangeAuthorization authorizedWhenInUse")
            // Stpe 6: Request a one-time location information
            locationManager.requestLocation()
        case .authorizedAlways: // Setting option: Always
            print("LocationManager didChangeAuthorization authorizedAlways")
            // Stpe 6: Request a one-time location information
            locationManager.requestLocation()
            debugPrint("Starting significant location tracking from authorization status change delegate")
            locationManager.allowsBackgroundLocationUpdates=true
            locationManager.pausesLocationUpdatesAutomatically = false
            locationManager.desiredAccuracy = kCLLocationAccuracyBestForNavigation
            locationManager.startMonitoringSignificantLocationChanges()
        case .restricted: // Restricted by parental control
            print("LocationManager didChangeAuthorization restricted")
        default:
            print("LocationManager didChangeAuthorization")
        }
    }
    
    func application(_ application:UIApplication, willFinishLaunchingWithOptions launchOptions:[UIApplication.LaunchOptionsKey:Any]?) ->Bool{
        if let keys = launchOptions?.keys{
            if(keys.contains(.location)){
                locationManager = CLLocationManager()
                locationManager.delegate = self
                self.distanceFilter = 50
                self.updateDuration = 60000
                debugPrint("OS invoked package to deliver location in background")
                locationManager.allowsBackgroundLocationUpdates=true
                locationManager.distanceFilter = distanceFilter
                locationManager.activityType = .automotiveNavigation
                locationManager.desiredAccuracy = kCLLocationAccuracyNearestTenMeters
                locationManager.requestAlwaysAuthorization()
                
            }
        }
        return true
    }
    
    @objc func locationManager(_ manager: CLLocationManager, didUpdateLocations locations: [CLLocation]) {
        debugPrint("CONFIGURED ACCURACY: \(locationManager.desiredAccuracy)")
        locations.forEach { (location) in
            
            ActivityRecognitionWrapper.shared?.sendEvent(withName: "LOCATION_UPDATE", body: getTransformedLocationObject(location:location))
            if(CMMotionActivityManager.isActivityAvailable()){
                debugPrint("Starting motion updates for \(updateDuration) seconds" )
                if(!activityServiceRunning){
                    debugPrint("activityServiceRunning is true")
                    activityServiceRunning = true
                    motionActivityManager.startActivityUpdates(to: OperationQueue.main)
                    { [self]
                        (activity) in
                        if (activity?.automotive)! {
                            ActivityRecognitionWrapper.shared?.sendEvent(withName: "ACTIVITY_UPDATE",  body: getTransformedActivityObject(activity: activity!, type: "IN_VEHICLE"))
                            //                        debugPrint("User using car")
                        }
                        if (activity?.cycling)! {
                            ActivityRecognitionWrapper.shared?.sendEvent(withName: "ACTIVITY_UPDATE", body: getTransformedActivityObject(activity: activity!, type: "CYCLING"))
                            //                        debugPrint("User is cycling")
                        }
                        if (activity?.running)!{
                            ActivityRecognitionWrapper.shared?.sendEvent(withName: "ACTIVITY_UPDATE", body: getTransformedActivityObject(activity: activity!, type: "RUNNING"))
                            //                        debugPrint("User is running")
                        }
                        if (activity?.walking)! {
                            ActivityRecognitionWrapper.shared?.sendEvent(withName: "ACTIVITY_UPDATE", body: getTransformedActivityObject(activity: activity!, type: "WALKING"))
                            //                        debugPrint("User is walking")
                        }
                        if (activity?.stationary)! {
                            ActivityRecognitionWrapper.shared?.sendEvent(withName: "ACTIVITY_UPDATE", body: getTransformedActivityObject(activity: activity!, type: "STILL"))
                            //                        debugPrint("User is stationary")
                        }
                        if (activity?.unknown)!{
                            ActivityRecognitionWrapper.shared?.sendEvent(withName: "ACTIVITY_UPDATE", body: getTransformedActivityObject(activity: activity!, type: "UNKNOWN"))
                            //                        debugPrint("Unknown activity")
                        }
                        
                        
                    }
                    DispatchQueue.main.asyncAfter(deadline: .now() + DispatchTimeInterval.milliseconds(updateDuration)) { [self] in
                        motionActivityManager.stopActivityUpdates()
                        locationManager.requestLocation()
                        debugPrint("Stopped motion updates after \(updateDuration) seconds" )
                        activityServiceRunning = false
                    }
                }
                
                
            }else {
                debugPrint("Motion not available")
            }
            
        }
    }
    
    @objc func locationManager(_ manager: CLLocationManager, didFailWithError error: Error) {
        debugPrint("LocationManager didFailWithError \(error.localizedDescription)")
        if let error = error as? CLError, error.code == .denied {
            // Location updates are not authorized.
            // To prevent forever looping of `didFailWithError` callback
            locationManager.stopMonitoringSignificantLocationChanges()
            locationManager.stopUpdatingLocation()
            
            return
        }
    }
    
}
